library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity CONV_ACCELERATOR_APPROXIMATESKIP is  -- (FOR 128x128 PIXELS)
generic (Q: integer:=27; -- 27
         L: integer:=18; --24
         P: integer:=16; -- 16
         J: integer:= 6;
         U: integer:= 4;
 	 R: integer:=8; --8
         H: integer:= 16; --16
	 O: integer:= 128 -- 128
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     START: in std_logic;
     PIXELS: in std_logic_vector(Q-1 downto 0);
     WEIGHT0: in std_logic_vector(L-1 downto 0);
     WEIGHT1: in std_logic_vector(L-1 downto 0); 
     WEIGHT2: in std_logic_vector(L-1 downto 0);
     WEIGHT3: in std_logic_vector(L-1 downto 0);
     WEIGHT4: in std_logic_vector(L-1 downto 0);
     WEIGHT5: in std_logic_vector(L-1 downto 0);
     WEIGHT6: in std_logic_vector(L-1 downto 0);
     WEIGHT7: in std_logic_vector(L-1 downto 0);
     WEIGHT8: in std_logic_vector(L-1 downto 0);
     TH: in std_logic_vector(L-1 downto 0);
     OM0WE: out std_logic;
     OM0OE: out std_logic;
     OM1WE: out std_logic;
     OM1OE: out std_logic;
     OM0_INPUT_CHANNEL1: in std_logic_vector(H-1 downto 0);
     OM1_INPUT_CHANNEL1: in std_logic_vector(H-1 downto 0);
     OM0_OUTPUT_CHANNEL1: out std_logic_vector(H-1 downto 0);
     OM1_OUTPUT_CHANNEL1: out std_logic_vector(H-1 downto 0);
     OM0_INPUT_CHANNEL2: in std_logic_vector(H-1 downto 0);
     OM1_INPUT_CHANNEL2: in std_logic_vector(H-1 downto 0);
     OM0_OUTPUT_CHANNEL2: out std_logic_vector(H-1 downto 0);
     OM1_OUTPUT_CHANNEL2: out std_logic_vector(H-1 downto 0);
     OM0_INPUT_CHANNEL3: in std_logic_vector(H-1 downto 0);
     OM1_INPUT_CHANNEL3: in std_logic_vector(H-1 downto 0);
     OM0_OUTPUT_CHANNEL3: out std_logic_vector(H-1 downto 0);
     OM1_OUTPUT_CHANNEL3: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL1: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL2: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL3: out std_logic_vector(H-1 downto 0));
end CONV_ACCELERATOR_APPROXIMATESKIP;

architecture structural of CONV_ACCELERATOR_APPROXIMATESKIP is

component DATAPATH_APPROXIMATESKIP is  -- (FOR 128x128 PIXELS)
generic (Q: integer:=27; --27
         L: integer:=18; -- 24
         P: integer:=16; -- 16
         J: integer:= 6;
         U: integer:= 4;
 	 R: integer:=9; --8
         H: integer:= 16; --16
	 O: integer:= 128 -- 131
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     PIXELS: in std_logic_vector(Q-1 downto 0);
     WEIGHT0: in std_logic_vector(L-1 downto 0);
     WEIGHT1: in std_logic_vector(L-1 downto 0); 
     WEIGHT2: in std_logic_vector(L-1 downto 0);
     WEIGHT3: in std_logic_vector(L-1 downto 0);
     WEIGHT4: in std_logic_vector(L-1 downto 0);
     WEIGHT5: in std_logic_vector(L-1 downto 0);
     WEIGHT6: in std_logic_vector(L-1 downto 0);
     WEIGHT7: in std_logic_vector(L-1 downto 0);
     WEIGHT8: in std_logic_vector(L-1 downto 0);
     TH: in std_logic_vector(L-1 downto 0);
     REGISTERS_ENABLE: in std_logic;
     OM0_INPUT_CHANNEL1: in std_logic_vector(H-1 downto 0);
     OM1_INPUT_CHANNEL1: in std_logic_vector(H-1 downto 0);
     OM0_OUTPUT_CHANNEL1: out std_logic_vector(H-1 downto 0);
     OM1_OUTPUT_CHANNEL1: out std_logic_vector(H-1 downto 0);
     OM0_INPUT_CHANNEL2: in std_logic_vector(H-1 downto 0);
     OM1_INPUT_CHANNEL2: in std_logic_vector(H-1 downto 0);
     OM0_OUTPUT_CHANNEL2: out std_logic_vector(H-1 downto 0);
     OM1_OUTPUT_CHANNEL2: out std_logic_vector(H-1 downto 0);
     OM0_INPUT_CHANNEL3: in std_logic_vector(H-1 downto 0);
     OM1_INPUT_CHANNEL3: in std_logic_vector(H-1 downto 0);
     OM0_OUTPUT_CHANNEL3: out std_logic_vector(H-1 downto 0);
     OM1_OUTPUT_CHANNEL3: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL1: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL2: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL3: out std_logic_vector(H-1 downto 0)); 
end component;

component ACCELERATOR_CONTROLLER_APPROXIMATESKIP is  -- (FOR 128x128 PIXELS)
generic (M: integer:=128; --128
         l: integer:=126; --126
         mm: integer:=125; --125
         n1: integer:=9; -- 8
         K1: integer:=16136; --16134
         n2: integer:=139; -- 138
         K2: integer:=16266; -- 16264
         n3: integer:=143; -- 142
         K3: integer:=16270; -- 16268
         n4: integer:=273 -- 272
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     START: in std_logic;
     REGISTERSOE: out std_logic;
     OM0WE: out std_logic;
     OM0OE: out std_logic;
     OM1WE: out std_logic;
     OM1OE: out std_logic); 
end component;

signal REG_E: std_logic;

begin

UUT_CONTROLLER: ACCELERATOR_CONTROLLER_APPROXIMATESKIP port map (CLK, RESET, START, REG_E, OM0WE, OM0OE, OM1WE, OM1OE);

UUT_DATAPATH: DATAPATH_APPROXIMATESKIP generic map (Q=>Q, L=>L, P=>P,J=>J,U=>U, R=>R, H=>H, O=>O) port map (CLK, RESET,  PIXELS, WEIGHT0, WEIGHT1, WEIGHT2, WEIGHT3, WEIGHT4, WEIGHT5, WEIGHT6, WEIGHT7, WEIGHT8,TH, REG_E, OM0_INPUT_CHANNEL1, OM1_INPUT_CHANNEL1, OM0_OUTPUT_CHANNEL1, OM1_OUTPUT_CHANNEL1, OM0_INPUT_CHANNEL2, OM1_INPUT_CHANNEL2, OM0_OUTPUT_CHANNEL2, OM1_OUTPUT_CHANNEL2, OM0_INPUT_CHANNEL3, OM1_INPUT_CHANNEL3, OM0_OUTPUT_CHANNEL3, OM1_OUTPUT_CHANNEL3, MAP_RESULT_CHANNEL1, MAP_RESULT_CHANNEL2, MAP_RESULT_CHANNEL3); 

end structural;
