library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity DATAPATH_APPROXIMATESKIP_SINGLECHANNEL is  -- (FOR 128x128 PIXELS)
generic (K: integer:=8; -- 24
         W: integer:= 6; 
         S: integer:= 4;
         G: integer:=16; -- 16
 	 reg1: integer:=9; --8
         reg2: integer:= 16; --16
	 MEM: integer:= 128 -- 128
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     PIXELS: in std_logic_vector(K downto 0);
     WEIGHT0: in std_logic_vector(W-1 downto 0);
     WEIGHT1: in std_logic_vector(W-1 downto 0); 
     WEIGHT2: in std_logic_vector(W-1 downto 0);
     WEIGHT3: in std_logic_vector(W-1 downto 0);
     WEIGHT4: in std_logic_vector(W-1 downto 0);
     WEIGHT5: in std_logic_vector(W-1 downto 0);
     WEIGHT6: in std_logic_vector(W-1 downto 0);
     WEIGHT7: in std_logic_vector(W-1 downto 0);
     WEIGHT8: in std_logic_vector(W-1 downto 0);
     TH: in std_logic_vector(W-1 downto 0);
     REGISTERS_ENABLE: in std_logic;
     OM_0_WRITE_ENABLE: in std_logic;
     OM_0_READ_ENABLE: in std_logic;
     OM_1_WRITE_ENABLE: in std_logic;
     OM_1_READ_ENABLE: in std_logic;
     MAP_RESULT: out std_logic_vector(G-1 downto 0));
end DATAPATH_APPROXIMATESKIP_SINGLECHANNEL ;    

architecture structural of DATAPATH_APPROXIMATESKIP_SINGLECHANNEL  is 

component PE_APPSKIPPING is
generic (n: integer; -- 9
         m: integer); --16
port( CLK: in std_logic;
      RESET: in std_logic;
      ZERO_EN: in std_logic;
      ACTIVATION: in std_logic_vector(n-1 downto 0);
      WEIGHT: in std_logic_vector(n-4 downto 0);
      PARTIAL_SUM: in std_logic_vector(m-1 downto 0);
      PARTIAL_RESULT: out std_logic_vector(m-1 downto 0));
end component;

component REGISTER_1bit is
port ( CLK: in std_logic;
       RESET: in std_logic;
       ENABLE: in std_logic;	   
       D : in std_logic;
       Q : out std_logic);
end component;

component REGISTER_nbit is
generic (n: integer);
port ( CLK: in std_logic;
       RESET: in std_logic;
       ENABLE: in std_logic;	   
       D : in std_logic_vector (n-1 downto 0);
       Q : out std_logic_vector (n-1 downto 0));
end component;

component LEADING_ZEROS_RECOGNIZER is
generic (n: integer; -- 9
         k: integer; -- 4
         m: integer); -- 6
port( CLK: in std_logic;
      RESET: in std_logic;
      ENABLE: in std_logic;
      ACTIVATION: in std_logic_vector(n-1 downto 0);
      WEIGHT: in std_logic_vector(n-4 downto 0);
      THRESHOLD_ZEROS: in std_logic_vector(m-1 downto 0);
      APPROXIMATE: out std_logic);
end component;

component ON_CHIP_MEMORY_RAM is
generic (m: integer; n: integer);
port(CLK: in std_logic;
     RESET: in std_logic;
     WE: in std_logic;
     OE: in std_logic;
	 DATA_INPUT: in std_logic_vector(n-1 downto 0);
     DATA_OUTPUT: out std_logic_vector(n-1 downto 0));
end component;   

-- WIRES 
signal REG1_REG2_LZREC0: std_logic_vector(reg1-1 downto 0);
signal REG2_REG3_PE0: std_logic_vector(reg1-1 downto 0);
signal REG3_REG4_LZREC1: std_logic_vector(reg1-1 downto 0);
signal REG4_REG5_PE1: std_logic_vector(reg1-1 downto 0);
signal REG5_REG6_LZREC2: std_logic_vector(reg1-1 downto 0);
signal REG6_REG7_PE2: std_logic_vector(reg1-1 downto 0);
signal REG7_REG8_LZREC3: std_logic_vector(reg1-1 downto 0);
signal REG8_REG9_PE3: std_logic_vector(reg1-1 downto 0);
signal REG9_REG10_LZREC4: std_logic_vector(reg1-1 downto 0);
signal REG10_REG11_PE4: std_logic_vector(reg1-1 downto 0);
signal REG11_REG12_LZREC5: std_logic_vector(reg1-1 downto 0);
signal REG12_REG13_PE5: std_logic_vector(reg1-1 downto 0);
signal REG13_REG14_LZREC6: std_logic_vector(reg1-1 downto 0);
signal REG14_REG15_PE6: std_logic_vector(reg1-1 downto 0);
signal REG15_REG16_LZREC7: std_logic_vector(reg1-1 downto 0);
signal REG16_REG17_PE7: std_logic_vector(reg1-1 downto 0);
signal REG17_REG18_LZREC8: std_logic_vector(reg1-1 downto 0);
signal REG18_PE8: std_logic_vector(reg1-1 downto 0);

signal LZREC0_PE0: std_logic;
signal LZREC1_PE1: std_logic;
signal LZREC2_PE2: std_logic;
signal LZREC3_PE3: std_logic;
signal LZREC4_PE4: std_logic;
signal LZREC5_PE5: std_logic;
signal LZREC6_PE6: std_logic;
signal LZREC7_PE7: std_logic;
signal LZREC8_PE8: std_logic;

signal PE0_MACREG0: std_logic_vector(reg2-1 downto 0);
signal PE1_MACREG1: std_logic_vector(reg2-1 downto 0);
signal PE2_MACREG2: std_logic_vector(reg2-1 downto 0);
signal PE3_MACREG3: std_logic_vector(reg2-1 downto 0);
signal PE4_MACREG4: std_logic_vector(reg2-1 downto 0);
signal PE5_MACREG5: std_logic_vector(reg2-1 downto 0);
signal PE6_MACREG6: std_logic_vector(reg2-1 downto 0);
signal PE7_MACREG7: std_logic_vector(reg2-1 downto 0);
signal PE8_MACREG8: std_logic_vector(reg2-1 downto 0);

signal MACREG0_PE1: std_logic_vector(reg2-1 downto 0);
signal MACREG1_PE2: std_logic_vector(reg2-1 downto 0);
signal MACREG2_OM0: std_logic_vector(reg2-1 downto 0);
signal OM0_PE3: std_logic_vector(reg2-1 downto 0);
signal MACREG3_PE4: std_logic_vector(reg2-1 downto 0);
signal MACREG4_PE5: std_logic_vector(reg2-1 downto 0);
signal MACREG5_OM1: std_logic_vector(reg2-1 downto 0);
signal OM1_PE6: std_logic_vector(reg2-1 downto 0);              
signal MACREG6_PE7: std_logic_vector(reg2-1 downto 0);
signal MACREG7_PE8: std_logic_vector(reg2-1 downto 0);

begin

-- UUT CHANNEL 1

UUT_REG1: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, PIXELS, REG1_REG2_LZREC0); 

UUT_REG2: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG1_REG2_LZREC0, REG2_REG3_PE0);

UUT_REG3: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG2_REG3_PE0, REG3_REG4_LZREC1);

UUT_REG4: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG3_REG4_LZREC1, REG4_REG5_PE1);

UUT_REG5: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG4_REG5_PE1, REG5_REG6_LZREC2);

UUT_REG6: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG5_REG6_LZREC2, REG6_REG7_PE2);

UUT_REG7: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG6_REG7_PE2, REG7_REG8_LZREC3);

UUT_REG8: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG7_REG8_LZREC3, REG8_REG9_PE3);

UUT_REG9: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG8_REG9_PE3, REG9_REG10_LZREC4);

UUT_REG10: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG9_REG10_LZREC4, REG10_REG11_PE4);

UUT_REG11: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG10_REG11_PE4, REG11_REG12_LZREC5);

UUT_REG12: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG11_REG12_LZREC5, REG12_REG13_PE5);

UUT_REG13: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG12_REG13_PE5, REG13_REG14_LZREC6);

UUT_REG14: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG13_REG14_LZREC6, REG14_REG15_PE6);

UUT_REG15: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG14_REG15_PE6, REG15_REG16_LZREC7);

UUT_REG16: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG15_REG16_LZREC7, REG16_REG17_PE7);

UUT_REG17: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG16_REG17_PE7, REG17_REG18_LZREC8);

UUT_REG18: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG17_REG18_LZREC8, REG18_PE8);

UUT_LZREC0: LEADING_ZEROS_RECOGNIZER generic map(n=>reg1, k=>S, m=>W) port map (CLK, RESET, REGISTERS_ENABLE, REG1_REG2_LZREC0, WEIGHT2, TH, LZREC0_PE0);

UUT_LZREC1: LEADING_ZEROS_RECOGNIZER generic map(n=>reg1, k=>S, m=>W) port map (CLK, RESET, REGISTERS_ENABLE,REG3_REG4_LZREC1, WEIGHT1, TH, LZREC1_PE1);

UUT_LZREC2: LEADING_ZEROS_RECOGNIZER generic map(n=>reg1, k=>S, m=>W) port map (CLK, RESET, REGISTERS_ENABLE,REG5_REG6_LZREC2, WEIGHT0, TH,  LZREC2_PE2);

UUT_LZREC3: LEADING_ZEROS_RECOGNIZER generic map(n=>reg1, k=>S, m=>W) port map (CLK, RESET, REGISTERS_ENABLE,REG7_REG8_LZREC3, WEIGHT5, TH,  LZREC3_PE3);

UUT_LZREC4: LEADING_ZEROS_RECOGNIZER generic map(n=>reg1, k=>S, m=>W) port map (CLK, RESET, REGISTERS_ENABLE,REG9_REG10_LZREC4, WEIGHT4, TH, LZREC4_PE4);

UUT_LZREC5: LEADING_ZEROS_RECOGNIZER generic map(n=>reg1, k=>S, m=>W) port map (CLK, RESET, REGISTERS_ENABLE,REG11_REG12_LZREC5, WEIGHT3, TH,  LZREC5_PE5);

UUT_LZREC6: LEADING_ZEROS_RECOGNIZER generic map(n=>reg1, k=>S, m=>W) port map (CLK, RESET, REGISTERS_ENABLE,REG13_REG14_LZREC6, WEIGHT8, TH,  LZREC6_PE6);

UUT_LZREC7: LEADING_ZEROS_RECOGNIZER generic map(n=>reg1, k=>S, m=>W) port map (CLK, RESET, REGISTERS_ENABLE,REG15_REG16_LZREC7, WEIGHT7, TH,  LZREC7_PE7);

UUT_LZREC8: LEADING_ZEROS_RECOGNIZER generic map(n=>reg1, k=>S, m=>W) port map (CLK, RESET, REGISTERS_ENABLE,REG17_REG18_LZREC8, WEIGHT6, TH, LZREC8_PE8);


UUT_PE0: PE_APPSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, LZREC0_PE0, REG2_REG3_PE0, WEIGHT2, (others=>'0'), PE0_MACREG0);

UUT_PE1: PE_APPSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, LZREC1_PE1, REG4_REG5_PE1, WEIGHT1, MACREG0_PE1, PE1_MACREG1);

UUT_PE2: PE_APPSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, LZREC2_PE2, REG6_REG7_PE2, WEIGHT0, MACREG1_PE2, PE2_MACREG2);

UUT_PE3: PE_APPSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, LZREC3_PE3, REG8_REG9_PE3, WEIGHT5, OM0_PE3, PE3_MACREG3);

UUT_PE4: PE_APPSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, LZREC4_PE4, REG10_REG11_PE4, WEIGHT4, MACREG3_PE4, PE4_MACREG4);

UUT_PE5: PE_APPSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, LZREC5_PE5, REG12_REG13_PE5, WEIGHT3, MACREG4_PE5, PE5_MACREG5);

UUT_PE6: PE_APPSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, LZREC6_PE6, REG14_REG15_PE6, WEIGHT8, OM1_PE6, PE6_MACREG6);

UUT_PE7: PE_APPSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, LZREC7_PE7, REG16_REG17_PE7, WEIGHT7, MACREG6_PE7, PE7_MACREG7);

UUT_PE8: PE_APPSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, LZREC8_PE8, REG18_PE8, WEIGHT6, MACREG7_PE8, PE8_MACREG8);


UUT_MACREG0: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE0_MACREG0, MACREG0_PE1);

UUT_MACREG1: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE1_MACREG1, MACREG1_PE2);

UUT_MACREG2: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE2_MACREG2, MACREG2_OM0);

UUT_MACREG3: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE3_MACREG3, MACREG3_PE4);

UUT_MACREG4: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE4_MACREG4, MACREG4_PE5);

UUT_MACREG5: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE5_MACREG5, MACREG5_OM1);

UUT_MACREG6: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE6_MACREG6, MACREG6_PE7);

UUT_MACREG7: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE7_MACREG7, MACREG7_PE8);

UUT_MACREG8: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE8_MACREG8,MAP_RESULT);

UUT_OM_0: ON_CHIP_MEMORY_RAM generic map (m=>MEM, n=>reg2) port map(CLK, RESET, OM_0_WRITE_ENABLE, OM_0_READ_ENABLE, MACREG2_OM0, OM0_PE3);

UUT_OM_1: ON_CHIP_MEMORY_RAM generic map (m=>MEM, n=>reg2) port map(CLK, RESET, OM_1_WRITE_ENABLE, OM_1_READ_ENABLE, MACREG5_OM1, OM1_PE6);

end structural;