library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;   
use ieee.std_logic_signed.all;

entity ON_CHIP_MEMORY_RAM is
generic (m: integer; --128 
         n: integer);
port(CLK: in std_logic;
     RESET: in std_logic;
     WE: in std_logic;
     OE: in std_logic;
     DATA_INPUT: in std_logic_vector(n-1 downto 0);
     DATA_OUTPUT: out std_logic_vector(n-1 downto 0));
end ON_CHIP_MEMORY_RAM;   

architecture behavioural of ON_CHIP_MEMORY_RAM is

type RAM_ARRAY is array (0 to m-1) of std_logic_vector(n-1 downto 0);        

signal RAM: RAM_ARRAY;
signal address1 : integer range 0 to m;
signal address1_1 : integer range 0 to m;
signal address2 : integer range 0 to m;
signal address2_2 : integer range 0 to m;

begin


-- writing process into ON CHIP MEMORY (if CONTROL UNIT activates WE)

write_process: process (CLK, RESET)

begin
if RESET = '1' then  
   address1 <= 0;
   address1_1 <= 0;
   RAM<=(others=>(others=>'0'));
   
   elsif (CLK'event and CLK = '1') then
          if WE = '1' then 
		     if (address1 < m) then
                         RAM(address1) <= DATA_INPUT;
                         address1<= address1 +1;
                         else
                         address1<= 1;
                         RAM(address1_1) <= DATA_INPUT;
                         end if;
		  end if;
                
                  end if;
end process write_process;

-- reading process from ON CHIP MEMORY( Partial result read if CONTROL UNIT activates OE)


	read_process: process (CLK,RESET)  

	
	begin
		if RESET = '1' then
			 address2 <= 0;
                         address2_2 <= 0;
			 DATA_OUTPUT <= (others=>'0');
		elsif (CLK'event and CLK = '1') then
               
			 if OE = '1' then
                       if (address2 < m) then
			     DATA_OUTPUT <= RAM(address2);	
                             address2 <= address2 + 1 ; 		
				 else 
		                address2<= 1;
                                DATA_OUTPUT <= RAM(address2_2);
			   end if;        
                                                  
        end if;
end if;    
	
    end process read_process;
       

end behavioural;
	