library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity ACCELERATOR_CONTROLLER_EWRSKIP is  -- (FOR 128x128 PIXELS)
generic (M: integer:=128; --128
         l: integer:=126; --126
         mm: integer:=125; --125
         n1: integer:=9; -- 8
         K1: integer:=16136; --16134
         n2: integer:=139; -- 138
         K2: integer:=16266; -- 16264
         n3: integer:=143; -- 142
         K3: integer:=16270; -- 16268
         n4: integer:=273; -- 272
         K4: integer:=16400 -- 16398
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     START: in std_logic;
     REGISTERSOE: out std_logic;
     OM0WE: out std_logic;
     OM0OE: out std_logic;
     OM1WE: out std_logic;
     OM1OE: out std_logic);
end ACCELERATOR_CONTROLLER_EWRSKIP ;

architecture behavioural of ACCELERATOR_CONTROLLER_EWRSKIP is

signal cnt1: integer;
signal cnt2: integer;
signal cnt3: integer;
signal cnt4: integer;
signal i: integer range n1 to (K1+2);
signal j: integer range n2 to (K2+2);
signal k: integer range n3 to (K3+2);
signal r: integer range n4 to (K4+2);

begin

----------------------------------------******
-- IDLE

IDLE_PROCESS: process(CLK,RESET)
begin
if RESET = '1' then
REGISTERSOE <= '0';

elsif CLK'event and CLK = '1' then
REGISTERSOE <= '1';
end if;
end process;
----------------------------------------******

----------------------------------------******
-- PROCESSING

-- OM_0 WRITING

OM_0_WRITING: process (CLK)
begin
if (CLK'event and CLK = '1') then
      if RESET = '1' then -- GO TO IDLE
         OM0WE <= '0';
         cnt1 <= 0;
         i<=n1;
      elsif START = '1' then  
           
             if cnt1 < K1 then 
	        
                if cnt1 >= i and cnt1 <= (i+mm) then
	           OM0WE <= '1';
		else
	           OM0WE <= '0';
                end if;
		
	           if cnt1 = (i+l) then
                      i<= i+M;
                   else
                      i<=i;
                   end if;
		
                   cnt1<= cnt1 +1;

              elsif cnt1 >= K1 then 
                    cnt1<=0;
                    OM0WE <= '0';
              end if;
       end if;
end if;
end process;

-- OM_0 READING

OM_0_READING: process (CLK)

begin

if (CLK'event and CLK = '1') then
        if RESET = '1' then -- GO TO IDLE
           OM0OE <= '0';
           cnt2<=0;
           j<=n2;
        elsif START = '1' then
      
               if cnt2 < K2 then  
	 
                   if cnt2 >= j and cnt2 <= (j+mm) then
	              OM0OE <= '1';
		   else
		      OM0OE <= '0';
                   end if;
		
	           if cnt2 = (j+l) then
                      j<= j+M;
                   else
                      j<=j;
                   end if;
        
                   cnt2<= cnt2 +1;

                elsif cnt2 >= K2 then
                      cnt2<=0;
                      OM0OE <= '0';
                end if;
       end if;
end if;
end process;

-- OM_1 WRITING

OM_1_WRITING: process (CLK)

begin

if (CLK'event and CLK = '1') then

         if RESET = '1' then -- GO TO IDLE
             OM1WE <= '0';
             cnt3<=0;
             k<=n3;
         elsif START = '1' then 
               
               if cnt3 < K3 then
	          
                   if cnt3 >= k and cnt3 <= (k+mm) then
	              OM1WE <= '1';
		   else
	              OM1WE <= '0';
                    end if;
		
	           if cnt3 = (k+l) then
                      k <= k+M;
                      else
                      k<=k;
                      end if;
                     
                      cnt3<=cnt3+1;
                
                elsif cnt3 >= K3 then
                       cnt3<=0;
                       OM1WE <= '0';
                end if;
         end if;
end if;
end process;

-- OM_1 READING

OM_1_READING: process (CLK)

begin

if (CLK'event and CLK = '1') then

             if RESET = '1' then -- GO TO IDLE
                OM1OE <= '0';
                cnt4<=0;
                r<=n4;
             elsif START = '1' then  
     
             if cnt4 < K4 then
	        
                if cnt4 >= r and cnt4 <= (r+mm) then
	           OM1OE <= '1';
		else
                   OM1OE <= '0';
                end if;
		
	        if cnt4 = (r+l) then
                   r<= r+M;
                else
                   r<=r;
                end if;
        
                cnt4<= cnt4+1;
             
             elsif cnt4 >= K4 then
                   cnt4 <=0; 
                  OM1OE <= '0';
             end if;
       end if;
end if;
end process;

----------------------------------------******

end behavioural;
	