library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity DATAPATH_EQUAL_WEIGHT_SKIP_MULTIPLICATION_SINGLECHANNEL is  -- (FOR 128x128 PIXELS)
generic (K: integer:=8; -- 8
         G: integer:=16; -- 16
 	 reg1: integer:=9; --8
         reg2: integer:= 16; --16
	 MEM: integer:= 128 -- 131
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     PIXELS: in std_logic_vector(K downto 0);
     WEIGHT0: in std_logic_vector(K-3 downto 0);
     WEIGHT1: in std_logic_vector(K-3 downto 0); 
     WEIGHT2: in std_logic_vector(K-3 downto 0);
     WEIGHT3: in std_logic_vector(K-3 downto 0);
     WEIGHT4: in std_logic_vector(K-3 downto 0);
     WEIGHT5: in std_logic_vector(K-3 downto 0);
     WEIGHT6: in std_logic_vector(K-3 downto 0);
     WEIGHT7: in std_logic_vector(K-3 downto 0);
     WEIGHT8: in std_logic_vector(K-3 downto 0);
     REGISTERS_ENABLE: in std_logic;
     OM_0_INPUT: in std_logic_vector (reg2-1 downto 0);
     OM_1_INPUT: in std_logic_vector (reg2-1 downto 0);
     OM_0_OUTPUT: out std_logic_vector (reg2-1 downto 0);
     OM_1_OUTPUT: out std_logic_vector (reg2-1 downto 0);
     MAP_RESULT: out std_logic_vector(G-1 downto 0));
end DATAPATH_EQUAL_WEIGHT_SKIP_MULTIPLICATION_SINGLECHANNEL;

architecture behavioural of DATAPATH_EQUAL_WEIGHT_SKIP_MULTIPLICATION_SINGLECHANNEL is

component EQUAL_WEIGHTS_RECOGNIZER is
generic (n: integer); --8
port( WEIGHT1: in std_logic_vector(n-1 downto 0);
      WEIGHT2: in std_logic_vector(n-1 downto 0);
      ZERO: out std_logic);
end component ;

component MUX_2_TO_1 is
generic (m: integer:=9);
port(DATAIN1: in std_logic_vector(m-1 downto 0);
     DATAIN2: in std_logic_vector(m-1 downto 0);
     SEL: in std_logic;
     DATAOUT : out std_logic_vector(m downto 0));
end component;

component PE_EQUAL_WEIGHTS is
generic (n: integer; -- 8
         m: integer); --16
port( CLK: in std_logic;
      RESET: in std_logic;
      ZERO_EN: in std_logic;
      ACTIVATION: in std_logic_vector(n downto 0);
      WEIGHT: in std_logic_vector(n-4 downto 0);
      PARTIAL_SUM: in std_logic_vector(m-1 downto 0);
      PARTIAL_RESULT: out std_logic_vector(m-1 downto 0));
end component ;

component REGISTER_nbit is
generic (n: integer);
port ( CLK: in std_logic;
       RESET: in std_logic;
       ENABLE: in std_logic;	   
       D : in std_logic_vector (n-1 downto 0);
       Q : out std_logic_vector (n-1 downto 0));
end component;


-- WIRES 
signal REG1_REG2: std_logic_vector(reg1-1 downto 0);
signal REG2_REG3_PE0: std_logic_vector(reg1-1 downto 0);
signal REG3_REG4: std_logic_vector(reg1-1 downto 0);
signal REG4_REG5_PE1: std_logic_vector(reg1-1 downto 0);
signal REG5_REG6: std_logic_vector(reg1-1 downto 0);
signal REG6_REG7_PE2: std_logic_vector(reg1-1 downto 0);
signal REG7_REG8: std_logic_vector(reg1-1 downto 0);
signal REG8_REG9_PE3: std_logic_vector(reg1-1 downto 0);
signal REG9_REG10: std_logic_vector(reg1-1 downto 0);
signal REG10_REG11_PE4: std_logic_vector(reg1-1 downto 0);
signal REG11_REG12: std_logic_vector(reg1-1 downto 0);
signal REG12_REG13_PE5: std_logic_vector(reg1-1 downto 0);
signal REG13_REG14: std_logic_vector(reg1-1 downto 0);
signal REG14_REG15_PE6: std_logic_vector(reg1-1 downto 0);
signal REG15_REG16: std_logic_vector(reg1-1 downto 0);
signal REG16_REG17_PE7: std_logic_vector(reg1-1 downto 0);
signal REG17_REG18: std_logic_vector(reg1-1 downto 0);
signal REG18_PE8: std_logic_vector(reg1-1 downto 0);

signal INPUT_PE2: std_logic_vector(reg1 downto 0);
signal INPUT_PE5: std_logic_vector(reg1 downto 0);
signal INPUT_PE8: std_logic_vector(reg1 downto 0);

signal CONTROL_EWR1_MUX0_PE1_PE2: std_logic;
signal CONTROL_EWR2_MUX1_PE2: std_logic;
signal CONTROL_EWR3_MUX2_PE4_PE5: std_logic;
signal CONTROL_EWR4_MUX3_PE5: std_logic;
signal CONTROL_EWR5_MUX4_PE7_PE8: std_logic;
signal CONTROL_EWR6_MUX5_PE8: std_logic;

signal MUX0_PE0: std_logic_vector(reg1 downto 0);
signal MUX1_PE1: std_logic_vector(reg1 downto 0);
signal MUX2_PE3: std_logic_vector(reg1 downto 0);
signal MUX3_PE4: std_logic_vector(reg1 downto 0);
signal MUX4_PE6: std_logic_vector(reg1 downto 0);
signal MUX5_PE7: std_logic_vector(reg1 downto 0);

signal ENABLE_P2: std_logic;
signal ENABLE_P5: std_logic;
signal ENABLE_P8: std_logic;

signal PE0_MACREG0: std_logic_vector(reg2-1 downto 0);
signal PE1_MACREG1: std_logic_vector(reg2-1 downto 0);
signal PE2_MACREG2: std_logic_vector(reg2-1 downto 0);
signal PE3_MACREG3: std_logic_vector(reg2-1 downto 0);
signal PE4_MACREG4: std_logic_vector(reg2-1 downto 0);
signal PE5_MACREG5: std_logic_vector(reg2-1 downto 0);
signal PE6_MACREG6: std_logic_vector(reg2-1 downto 0);
signal PE7_MACREG7: std_logic_vector(reg2-1 downto 0);
signal PE8_MACREG8: std_logic_vector(reg2-1 downto 0);

signal MACREG0_PE1: std_logic_vector(reg2-1 downto 0);
signal MACREG1_PE2: std_logic_vector(reg2-1 downto 0);
--signal MACREG2_OM0: std_logic_vector(reg2-1 downto 0);
--signal OM0_PE3: std_logic_vector(reg2-1 downto 0);
signal MACREG3_PE4: std_logic_vector(reg2-1 downto 0);
signal MACREG4_PE5: std_logic_vector(reg2-1 downto 0);
--signal MACREG5_OM1: std_logic_vector(reg2-1 downto 0);
--signal OM1_PE6: std_logic_vector(reg2-1 downto 0);
signal MACREG6_PE7: std_logic_vector(reg2-1 downto 0);
signal MACREG7_PE8: std_logic_vector(reg2-1 downto 0);

begin

-- UUT SINGLE CHANNEL 

UUT_REG1: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, PIXELS, REG1_REG2); 

UUT_REG2: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG1_REG2, REG2_REG3_PE0);

UUT_REG3: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG2_REG3_PE0, REG3_REG4);

UUT_REG4: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG3_REG4, REG4_REG5_PE1);

UUT_REG5: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG4_REG5_PE1, REG5_REG6);

UUT_REG6: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG5_REG6, REG6_REG7_PE2);

UUT_REG7: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG6_REG7_PE2, REG7_REG8);

UUT_REG8: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG7_REG8, REG8_REG9_PE3);

UUT_REG9: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG8_REG9_PE3, REG9_REG10);

UUT_REG10: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG9_REG10, REG10_REG11_PE4);

UUT_REG11: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG10_REG11_PE4, REG11_REG12);

UUT_REG12: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG11_REG12, REG12_REG13_PE5);

UUT_REG13: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG12_REG13_PE5, REG13_REG14);

UUT_REG14: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG13_REG14, REG14_REG15_PE6);

UUT_REG15: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG14_REG15_PE6, REG15_REG16);

UUT_REG16: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG15_REG16, REG16_REG17_PE7);

UUT_REG17: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG16_REG17_PE7, REG17_REG18);

UUT_REG18: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG17_REG18, REG18_PE8);

INPUT_PE2 <= '0'&REG6_REG7_PE2;
INPUT_PE5 <= '0'&REG12_REG13_PE5;
INPUT_PE8 <= '0'&REG18_PE8;

UUT_EWR1: EQUAL_WEIGHTS_RECOGNIZER generic map (n=>K-2) port map (WEIGHT2, WEIGHT1, CONTROL_EWR1_MUX0_PE1_PE2);

UUT_EWR2: EQUAL_WEIGHTS_RECOGNIZER generic map (n=>K-2) port map (WEIGHT1, WEIGHT0, CONTROL_EWR2_MUX1_PE2);

UUT_EWR3: EQUAL_WEIGHTS_RECOGNIZER generic map (n=>K-2) port map (WEIGHT5, WEIGHT4, CONTROL_EWR3_MUX2_PE4_PE5);

UUT_EWR4: EQUAL_WEIGHTS_RECOGNIZER generic map (n=>K-2) port map (WEIGHT4, WEIGHT3, CONTROL_EWR4_MUX3_PE5);

UUT_EWR5: EQUAL_WEIGHTS_RECOGNIZER generic map (n=>K-2) port map (WEIGHT8, WEIGHT7, CONTROL_EWR5_MUX4_PE7_PE8);

UUT_EWR6: EQUAL_WEIGHTS_RECOGNIZER generic map (n=>K-2) port map (WEIGHT7, WEIGHT6, CONTROL_EWR6_MUX5_PE8);


UUT_MUX0: MUX_2_TO_1 generic map(m=>reg1) port map (REG2_REG3_PE0, REG3_REG4, CONTROL_EWR1_MUX0_PE1_PE2, MUX0_PE0);

UUT_MUX1: MUX_2_TO_1 generic map(m=>reg1) port map (REG4_REG5_PE1, REG5_REG6, CONTROL_EWR2_MUX1_PE2, MUX1_PE1);

UUT_MUX2: MUX_2_TO_1 generic map(m=>reg1) port map (REG8_REG9_PE3, REG9_REG10, CONTROL_EWR3_MUX2_PE4_PE5, MUX2_PE3);

UUT_MUX3: MUX_2_TO_1 generic map(m=>reg1) port map (REG10_REG11_PE4, REG11_REG12, CONTROL_EWR4_MUX3_PE5, MUX3_PE4);

UUT_MUX4: MUX_2_TO_1 generic map( m=>reg1) port map (REG14_REG15_PE6, REG15_REG16, CONTROL_EWR5_MUX4_PE7_PE8, MUX4_PE6);

UUT_MUX5: MUX_2_TO_1 generic map(m=>reg1) port map (REG16_REG17_PE7, REG17_REG18, CONTROL_EWR6_MUX5_PE8, MUX5_PE7);

UUT_PE0: PE_EQUAL_WEIGHTS  generic map(n=>reg1, m=>reg2) port map(CLK, RESET, '1', MUX0_PE0, WEIGHT2, "0000000000000000", PE0_MACREG0);

UUT_PE1: PE_EQUAL_WEIGHTS  generic map(n=>reg1, m=>reg2) port map(CLK, RESET, CONTROL_EWR1_MUX0_PE1_PE2, MUX1_PE1, WEIGHT1, MACREG0_PE1, PE1_MACREG1);

ENABLE_P2 <= CONTROL_EWR2_MUX1_PE2 or (not(CONTROL_EWR1_MUX0_PE1_PE2));

UUT_PE2: PE_EQUAL_WEIGHTS  generic map(n=>reg1, m=>reg2) port map(CLK, RESET, ENABLE_P2, INPUT_PE2, WEIGHT0, MACREG1_PE2, PE2_MACREG2);

UUT_PE3: PE_EQUAL_WEIGHTS  generic map(n=>reg1, m=>reg2) port map(CLK, RESET, '1', MUX2_PE3, WEIGHT5, OM_0_INPUT, PE3_MACREG3);

UUT_PE4: PE_EQUAL_WEIGHTS  generic map(n=>reg1, m=>reg2) port map(CLK, RESET, CONTROL_EWR3_MUX2_PE4_PE5, MUX3_PE4,  WEIGHT4, MACREG3_PE4, PE4_MACREG4);

ENABLE_P5 <= CONTROL_EWR4_MUX3_PE5 or (not(CONTROL_EWR3_MUX2_PE4_PE5)); 

UUT_PE5: PE_EQUAL_WEIGHTS  generic map(n=>reg1,  m=>reg2) port map(CLK, RESET,ENABLE_P5, INPUT_PE5, WEIGHT3, MACREG4_PE5, PE5_MACREG5);

UUT_PE6: PE_EQUAL_WEIGHTS  generic map(n=>reg1,  m=>reg2) port map(CLK, RESET, '1', MUX4_PE6, WEIGHT8, OM_1_INPUT, PE6_MACREG6);

UUT_PE7: PE_EQUAL_WEIGHTS  generic map(n=>reg1, m=>reg2) port map(CLK, RESET, CONTROL_EWR5_MUX4_PE7_PE8,  MUX5_PE7, WEIGHT7, MACREG6_PE7, PE7_MACREG7);

ENABLE_P8 <= CONTROL_EWR6_MUX5_PE8 or (not(CONTROL_EWR5_MUX4_PE7_PE8));

UUT_PE8: PE_EQUAL_WEIGHTS  generic map(n=>reg1,  m=>reg2) port map(CLK, RESET, ENABLE_P8, INPUT_PE8, WEIGHT6, MACREG7_PE8, PE8_MACREG8);


UUT_MACREG0: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE0_MACREG0,MACREG0_PE1);

UUT_MACREG1: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE1_MACREG1,MACREG1_PE2);

UUT_MACREG2: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE2_MACREG2,OM_0_OUTPUT);

UUT_MACREG3: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE3_MACREG3,MACREG3_PE4);

UUT_MACREG4: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE4_MACREG4,MACREG4_PE5);

UUT_MACREG5: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE5_MACREG5,OM_1_OUTPUT);

UUT_MACREG6: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE6_MACREG6,MACREG6_PE7);

UUT_MACREG7: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE7_MACREG7,MACREG7_PE8);

UUT_MACREG8: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE8_MACREG8,MAP_RESULT);


end behavioural;
