library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;

entity PE_EQUAL_WEIGHTS is
generic (n: integer; -- 9
         m: integer); --16
port( CLK: in std_logic;
      RESET: in std_logic;
      ZERO_EN: in std_logic;
      ACTIVATION: in std_logic_vector(n downto 0);
      WEIGHT: in std_logic_vector(n-4 downto 0);
      PARTIAL_SUM: in std_logic_vector(m-1 downto 0);
      PARTIAL_RESULT: out std_logic_vector(m-1 downto 0));
end PE_EQUAL_WEIGHTS ;

architecture structural of PE_EQUAL_WEIGHTS is

component REGISTER_nbit is
generic (n: integer);
port ( CLK: in std_logic;
       RESET: in std_logic;
       ENABLE: in std_logic;	   
       D : in std_logic_vector (n-1 downto 0);
       Q : out std_logic_vector (n-1 downto 0));
end component;

component REGISTER_1bit is
port ( CLK: in std_logic;
       RESET: in std_logic;
       ENABLE: in std_logic;	   
       D : in std_logic;
       Q : out std_logic);
end component;

component PE_STANDARD_EWR is
generic (n: integer; -- 8
         m: integer); --16
port( ACTIVATION: in std_logic_vector(n downto 0);
      WEIGHT: in std_logic_vector(n-4 downto 0);
      ZERO: in std_logic;
      PARTIAL_SUM: in std_logic_vector(m-1 downto 0);
      PARTIAL_RESULT: out std_logic_vector(m-1 downto 0));
end component;


signal REG_ACT2: std_logic_vector(n downto 0);
signal REG_WEIGHT2: std_logic_vector(n-4 downto 0);
signal REG_MUX: std_logic;


begin

UUT_REG_A: REGISTER_nbit generic map (n=>10) port map (CLK, RESET, ZERO_EN, ACTIVATION, REG_ACT2);

UUT_REG_W: REGISTER_nbit generic map (n=>6) port map (CLK, RESET, ZERO_EN, WEIGHT, REG_WEIGHT2);

UUT_REG_MUX: REGISTER_1bit port map (CLK, RESET, '1', ZERO_EN, REG_MUX);

PE_UUT: PE_STANDARD_EWR generic map(n=>9, m=>16) port map(REG_ACT2, REG_WEIGHT2, REG_MUX, PARTIAL_SUM, PARTIAL_RESULT);

end structural; 

