library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity DATAPATH_HYBRID_EWS_APPS_MULTIPLICATION is  -- (FOR 128x128 PIXELS)
generic (Q: integer:=27; --27
         L: integer:=18; -- 24
         P: integer:=16; -- 16
         J: integer:= 6;
         U: integer:= 4;
 	 R: integer:=9; --8
         H: integer:= 16; --16
	 O: integer:= 128 -- 128
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     PIXELS: in std_logic_vector(Q-1 downto 0);
     WEIGHT0: in std_logic_vector(L-1 downto 0);
     WEIGHT1: in std_logic_vector(L-1 downto 0); 
     WEIGHT2: in std_logic_vector(L-1 downto 0);
     WEIGHT3: in std_logic_vector(L-1 downto 0);
     WEIGHT4: in std_logic_vector(L-1 downto 0);
     WEIGHT5: in std_logic_vector(L-1 downto 0);
     WEIGHT6: in std_logic_vector(L-1 downto 0);
     WEIGHT7: in std_logic_vector(L-1 downto 0);
     WEIGHT8: in std_logic_vector(L-1 downto 0);
     TH: in std_logic_vector(L-1 downto 0);
     REGISTERS_ENABLE: in std_logic;
     OM_0_WRITE_ENABLE: in std_logic;
     OM_0_READ_ENABLE: in std_logic;
     OM_1_WRITE_ENABLE: in std_logic;
     OM_1_READ_ENABLE: in std_logic;
     MAP_RESULT_CHANNEL1: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL2: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL3: out std_logic_vector(H-1 downto 0)); 
end DATAPATH_HYBRID_EWS_APPS_MULTIPLICATION ;

architecture structural of DATAPATH_HYBRID_EWS_APPS_MULTIPLICATION is

component DATAPATH_HYBRID_EWS_APPS_MULTIPLICATION_SINGLECHANNEL is  -- (FOR 128x128 PIXELS)
generic (K: integer:=8; -- 8
         G: integer:=16; -- 16
	 S: integer:=4;
	 W: integer:=6;
 	 reg1: integer:=9; --8
         reg2: integer:= 16; --16
	 MEM: integer:= 128 -- 131
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     PIXELS: in std_logic_vector(K downto 0);
     WEIGHT0: in std_logic_vector(K-3 downto 0);
     WEIGHT1: in std_logic_vector(K-3 downto 0); 
     WEIGHT2: in std_logic_vector(K-3 downto 0);
     WEIGHT3: in std_logic_vector(K-3 downto 0);
     WEIGHT4: in std_logic_vector(K-3 downto 0);
     WEIGHT5: in std_logic_vector(K-3 downto 0);
     WEIGHT6: in std_logic_vector(K-3 downto 0);
     WEIGHT7: in std_logic_vector(K-3 downto 0);
     WEIGHT8: in std_logic_vector(K-3 downto 0);
     TH: in std_logic_vector(K-3 downto 0);
     REGISTERS_ENABLE: in std_logic;
     OM_0_WRITE_ENABLE: in std_logic;
     OM_0_READ_ENABLE: in std_logic;
     OM_1_WRITE_ENABLE: in std_logic;
     OM_1_READ_ENABLE: in std_logic;
     MAP_RESULT: out std_logic_vector(G-1 downto 0));
end component;


begin

UUT_DATAPATH_CHANNEL1: DATAPATH_HYBRID_EWS_APPS_MULTIPLICATION_SINGLECHANNEL generic map (K=> R-1, G => P, S=>U, W=>J, reg1 => R, reg2 => H, MEM => O) port map (CLK, RESET,  PIXELS(26 downto 18), WEIGHT0(17 downto 12), WEIGHT1(17 downto 12), WEIGHT2(17 downto 12), WEIGHT3(17 downto 12), WEIGHT4(17 downto 12), WEIGHT5(17 downto 12), WEIGHT6(17 downto 12), WEIGHT7(17 downto 12), WEIGHT8(17 downto 12), TH(17 downto 12), REGISTERS_ENABLE,OM_0_WRITE_ENABLE, OM_0_READ_ENABLE, OM_1_WRITE_ENABLE, OM_1_READ_ENABLE, MAP_RESULT_CHANNEL1); 

UUT_DATAPATH_CHANNEL2: DATAPATH_HYBRID_EWS_APPS_MULTIPLICATION_SINGLECHANNEL generic map (K=> R-1, G => P, S=>U, W=>J, reg1 => R, reg2 => H, MEM => O) port map (CLK, RESET,  PIXELS(17 downto 9), WEIGHT0(11 downto 6), WEIGHT1(11 downto 6), WEIGHT2(11 downto 6), WEIGHT3(11 downto 6), WEIGHT4(11 downto 6), WEIGHT5(11 downto 6), WEIGHT6(11 downto 6), WEIGHT7(11 downto 6), WEIGHT8(11 downto 6), TH(11 downto 6), REGISTERS_ENABLE,OM_0_WRITE_ENABLE, OM_0_READ_ENABLE, OM_1_WRITE_ENABLE, OM_1_READ_ENABLE, MAP_RESULT_CHANNEL2);

UUT_DATAPATH_CHANNEL3: DATAPATH_HYBRID_EWS_APPS_MULTIPLICATION_SINGLECHANNEL generic map (K=> R-1, G => P, S=>U, W=>J, reg1 => R, reg2 => H, MEM => O) port map (CLK, RESET,  PIXELS(8 downto 0), WEIGHT0(5 downto 0), WEIGHT1(5 downto 0), WEIGHT2(5 downto 0), WEIGHT3(5 downto 0), WEIGHT4(5 downto 0), WEIGHT5(5 downto 0), WEIGHT6(5 downto 0), WEIGHT7(5 downto 0), WEIGHT8(5 downto 0), TH(5 downto 0), REGISTERS_ENABLE,OM_0_WRITE_ENABLE, OM_0_READ_ENABLE, OM_1_WRITE_ENABLE, OM_1_READ_ENABLE, MAP_RESULT_CHANNEL3);


end structural;