library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity LEADING_ZEROS_RECOGNIZER is
generic (n: integer:= 9; -- 8
         k: integer:= 4; -- 4
         m: integer:= 6); -- 6
port( CLK: in std_logic;
      RESET: in std_logic;
      ENABLE: in std_logic;
      ACTIVATION: in std_logic_vector(n downto 0);
      WEIGHT: in std_logic_vector(n-4 downto 0);
      THRESHOLD_ZEROS: in std_logic_vector(m-1 downto 0);
      APPROXIMATE: out std_logic);
end LEADING_ZEROS_RECOGNIZER;

architecture structural of LEADING_ZEROS_RECOGNIZER is

component LEADING_ZEROS_COUNTER_16bit_ACTS is
port( DIN: in std_logic_vector(9 downto 0);
      Z: out std_logic_vector(3 downto 0);
      V: out std_logic);
end component;

component LEADING_ZEROS_COUNTER_16bit_WEIGHTS is
port( DIN: in std_logic_vector(5 downto 0);
      Z: out std_logic_vector(3 downto 0);
      V: out std_logic);
end component;

component REGISTER_nbit is
generic (n: integer);
port ( CLK: in std_logic;
       RESET: in std_logic;
       ENABLE: in std_logic;	   
       D : in std_logic_vector (n-1 downto 0);
       Q : out std_logic_vector (n-1 downto 0));
end component;

component COMPARE_THRESHOLD is
generic ( n: integer; -- 5
          m: integer); -- 6
port( ACTIVATION_ZEROS: in std_logic_vector(n-1 downto 0);
      WEIGHT_ZEROS: in std_logic_vector(n-1 downto 0);
      THRESHOLD_ZEROS: in std_logic_vector(m-1 downto 0);
      APPROXIMATE: out std_logic);
end component;


signal LZ_ACT_COMPARE: std_logic_vector (k-1 downto 0);
signal LZ_WEIGHT_COMPARE: std_logic_vector (k-1 downto 0);
signal V_A: std_logic;
signal V_W: std_logic;
signal LZ_ACT_V: std_logic_vector (k downto 0);
signal LZ_WEIGHT_V: std_logic_vector (k downto 0);
signal REG_LZACT: std_logic_vector(k downto 0);
signal REG_LZW: std_logic_vector(k downto 0);

begin

--EXTEND THE SIGN OF THE THRESHOLD

UUT_LEADING_ZEROS_ACT: LEADING_ZEROS_COUNTER_16bit_ACTS port map ( ACTIVATION, LZ_ACT_COMPARE, V_A);
UUT_LEADING_ZEROS_WEIGHT: LEADING_ZEROS_COUNTER_16bit_WEIGHTS port map ( WEIGHT, LZ_WEIGHT_COMPARE, V_W);

LZ_ACT_V <= '0'& LZ_ACT_COMPARE;
LZ_WEIGHT_V <= '0'& LZ_WEIGHT_COMPARE;

UUT_REG_LZACT: REGISTER_nbit generic map (n => 5) port map (CLK,RESET,ENABLE, LZ_ACT_V, REG_LZACT);
UUT_REG_LZW: REGISTER_nbit generic map (n => 5) port map (CLK,RESET,ENABLE, LZ_WEIGHT_V, REG_LZW); 

UUT_COMPARE_THRESHOLD: COMPARE_THRESHOLD generic map (n=> 5, m=> 6) port map (REG_LZACT, REG_LZW, THRESHOLD_ZEROS, APPROXIMATE);

end structural;