library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity CONV_ACCELERATOR is  -- (FOR 128x128 PIXELS)
generic (Q: integer:=27; -- 27
         L: integer:=18; --24
         P: integer:=16; -- 16
 	 R: integer:=9; --9
         H: integer:= 16; --16
	 O: integer:= 128 -- 128
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     START: in std_logic;
     PIXELS: in std_logic_vector(Q-1 downto 0);
     WEIGHT0: in std_logic_vector(L-1 downto 0);
     WEIGHT1: in std_logic_vector(L-1 downto 0); 
     WEIGHT2: in std_logic_vector(L-1 downto 0);
     WEIGHT3: in std_logic_vector(L-1 downto 0);
     WEIGHT4: in std_logic_vector(L-1 downto 0);
     WEIGHT5: in std_logic_vector(L-1 downto 0);
     WEIGHT6: in std_logic_vector(L-1 downto 0);
     WEIGHT7: in std_logic_vector(L-1 downto 0);
     WEIGHT8: in std_logic_vector(L-1 downto 0);
     MAP_RESULT_CHANNEL1: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL2: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL3: out std_logic_vector(H-1 downto 0));
end CONV_ACCELERATOR;

architecture structural of CONV_ACCELERATOR is

component DATAPATH2 is  -- (FOR 128x128 PIXELS)
generic (Q: integer:=27; -- 27
         L: integer:=18; --18 
         P: integer:=16; -- 16
 	 R: integer:=9; --9
         H: integer:= 16; --16
	 O: integer:= 128 -- 131
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     PIXELS: in std_logic_vector(Q-1 downto 0);
     WEIGHT0: in std_logic_vector(L-1 downto 0);
     WEIGHT1: in std_logic_vector(L-1 downto 0); 
     WEIGHT2: in std_logic_vector(L-1 downto 0);
     WEIGHT3: in std_logic_vector(L-1 downto 0);
     WEIGHT4: in std_logic_vector(L-1 downto 0);
     WEIGHT5: in std_logic_vector(L-1 downto 0);
     WEIGHT6: in std_logic_vector(L-1 downto 0);
     WEIGHT7: in std_logic_vector(L-1 downto 0);
     WEIGHT8: in std_logic_vector(L-1 downto 0);
     REGISTERS_ENABLE: in std_logic;
     OM_0_WRITE_ENABLE: in std_logic;
     OM_0_READ_ENABLE: in std_logic;
     OM_1_WRITE_ENABLE: in std_logic;
     OM_1_READ_ENABLE: in std_logic;
     MAP_RESULT_CHANNEL1: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL2: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL3: out std_logic_vector(H-1 downto 0)); 
end component;

component ACCELERATOR_CONTROLLER is  -- (FOR 128x128 PIXELS)
generic (M: integer:=128; --128
         l: integer:=126; --126
         mm: integer:=125; --125
         n1: integer:=8; -- 8
         K1: integer:=16134; --16134
         n2: integer:=138; -- 138
         K2: integer:=16264; -- 16264
         n3: integer:=142; -- 142
         K3: integer:=16268; -- 16268
         n4: integer:=272; -- 272
         K4: integer:=16398 -- 16398
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     START: in std_logic;
     REGISTERSOE: out std_logic;
     OM0WE: out std_logic;
     OM0OE: out std_logic;
     OM1WE: out std_logic;
     OM1OE: out std_logic);
end component;

signal REG_E: std_logic;
signal OM0_W: std_logic;
signal OM0_R: std_logic;
signal OM1_W: std_logic;
signal OM1_R: std_logic;

begin

UUT_CONTROLLER: ACCELERATOR_CONTROLLER port map (CLK, RESET, START, REG_E, OM0_W, OM0_R, OM1_W, OM1_R);

UUT_DATAPATH: DATAPATH2 generic map (Q=>Q, L=>L, P=>P, R=>R, H=>H, O=>O) port map (CLK, RESET,  PIXELS, WEIGHT0, WEIGHT1, WEIGHT2, WEIGHT3, WEIGHT4, WEIGHT5, WEIGHT6, WEIGHT7, WEIGHT8, REG_E, OM0_W, OM0_R, OM1_W, OM1_R, MAP_RESULT_CHANNEL1, MAP_RESULT_CHANNEL2, MAP_RESULT_CHANNEL3); 

end structural;
