library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity DATAPATH is  -- (FOR 128x128 PIXELS)
generic (Q: integer:=27; -- 27
         L: integer:=18; --24
         P: integer:=16; -- 16
 	 R: integer:=9; --9
         H: integer:= 16; --16
	 O: integer:= 128 -- 131
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     PIXELS: in std_logic_vector(Q-1 downto 0);
     WEIGHT0: in std_logic_vector(L-1 downto 0);
     WEIGHT1: in std_logic_vector(L-1 downto 0); 
     WEIGHT2: in std_logic_vector(L-1 downto 0);
     WEIGHT3: in std_logic_vector(L-1 downto 0);
     WEIGHT4: in std_logic_vector(L-1 downto 0);
     WEIGHT5: in std_logic_vector(L-1 downto 0);
     WEIGHT6: in std_logic_vector(L-1 downto 0);
     WEIGHT7: in std_logic_vector(L-1 downto 0);
     WEIGHT8: in std_logic_vector(L-1 downto 0);
     REGISTERS_ENABLE: in std_logic;
     OM0_INPUT_CHANNEL1: in std_logic_vector(H-1 downto 0);
     OM1_INPUT_CHANNEL1: in std_logic_vector(H-1 downto 0);
     OM0_OUTPUT_CHANNEL1: out std_logic_vector(H-1 downto 0);
     OM1_OUTPUT_CHANNEL1: out std_logic_vector(H-1 downto 0);
     OM0_INPUT_CHANNEL2: in std_logic_vector(H-1 downto 0);
     OM1_INPUT_CHANNEL2: in std_logic_vector(H-1 downto 0);
     OM0_OUTPUT_CHANNEL2: out std_logic_vector(H-1 downto 0);
     OM1_OUTPUT_CHANNEL2: out std_logic_vector(H-1 downto 0);
     OM0_INPUT_CHANNEL3: in std_logic_vector(H-1 downto 0);
     OM1_INPUT_CHANNEL3: in std_logic_vector(H-1 downto 0);
     OM0_OUTPUT_CHANNEL3: out std_logic_vector(H-1 downto 0);
     OM1_OUTPUT_CHANNEL3: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL1: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL2: out std_logic_vector(H-1 downto 0);
     MAP_RESULT_CHANNEL3: out std_logic_vector(H-1 downto 0)); 
end DATAPATH ;

architecture structural of DATAPATH is

component DATAPATH_SINGLECHANNEL is  -- (FOR 128x128 PIXELS)
generic (D: integer:=9; --9
         K: integer:=6; -- 8
         G: integer:=16; -- 16
 	 reg1: integer:=9; --8
         reg2: integer:= 16; --16
	 MEM: integer:= 128 -- 131
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     PIXELS: in std_logic_vector(D-1 downto 0);
     WEIGHT0: in std_logic_vector(K-1 downto 0);
     WEIGHT1: in std_logic_vector(K-1 downto 0); 
     WEIGHT2: in std_logic_vector(K-1 downto 0);
     WEIGHT3: in std_logic_vector(K-1 downto 0);
     WEIGHT4: in std_logic_vector(K-1 downto 0);
     WEIGHT5: in std_logic_vector(K-1 downto 0);
     WEIGHT6: in std_logic_vector(K-1 downto 0);
     WEIGHT7: in std_logic_vector(K-1 downto 0);
     WEIGHT8: in std_logic_vector(K-1 downto 0);
     REGISTERS_ENABLE: in std_logic;
     OM_0_INPUT: in std_logic_vector(reg2-1 downto 0);
     OM_1_INPUT: in std_logic_vector(reg2-1 downto 0);
     OM_0_OUTPUT: out std_logic_vector(reg2-1 downto 0);
     OM_1_OUTPUT: out std_logic_vector(reg2-1 downto 0);
     MAP_RESULT: out std_logic_vector(G-1 downto 0));
end component;


begin

UUT_DATAPATH_CHANNEL1: DATAPATH_SINGLECHANNEL generic map (D=>R, K=> R-3, G => P,  reg1 => R, reg2 => H, MEM => O) port map (CLK, RESET,  PIXELS(26 downto 18), WEIGHT0(17 downto 12), WEIGHT1(17 downto 12), WEIGHT2(17 downto 12), WEIGHT3(17 downto 12), WEIGHT4(17 downto 12), WEIGHT5(17 downto 12), WEIGHT6(17 downto 12), WEIGHT7(17 downto 12), WEIGHT8(17 downto 12), REGISTERS_ENABLE, OM0_INPUT_CHANNEL1,OM1_INPUT_CHANNEL1,OM0_OUTPUT_CHANNEL1,OM1_OUTPUT_CHANNEL1, MAP_RESULT_CHANNEL1); 

UUT_DATAPATH_CHANNEL2: DATAPATH_SINGLECHANNEL generic map (D=>R, K=> R-3, G => P,  reg1 => R, reg2 => H, MEM => O) port map (CLK, RESET,  PIXELS(17 downto 9), WEIGHT0(11 downto 6), WEIGHT1(11 downto 6), WEIGHT2(11 downto 6), WEIGHT3(11 downto 6), WEIGHT4(11 downto 6), WEIGHT5(11 downto 6), WEIGHT6(11 downto 6), WEIGHT7(11 downto 6), WEIGHT8(11 downto 6), REGISTERS_ENABLE, OM0_INPUT_CHANNEL2,OM1_INPUT_CHANNEL2,OM0_OUTPUT_CHANNEL2,OM1_OUTPUT_CHANNEL2, MAP_RESULT_CHANNEL2);

UUT_DATAPATH_CHANNEL3: DATAPATH_SINGLECHANNEL generic map (D=>R, K=> R-3, G => P,  reg1 => R, reg2 => H, MEM => O) port map (CLK, RESET,  PIXELS(8 downto 0), WEIGHT0(5 downto 0), WEIGHT1(5 downto 0), WEIGHT2(5 downto 0), WEIGHT3(5 downto 0), WEIGHT4(5 downto 0), WEIGHT5(5 downto 0), WEIGHT6(5 downto 0), WEIGHT7(5 downto 0), WEIGHT8(5 downto 0), REGISTERS_ENABLE,OM0_INPUT_CHANNEL3,OM1_INPUT_CHANNEL3,OM0_OUTPUT_CHANNEL3,OM1_OUTPUT_CHANNEL3, MAP_RESULT_CHANNEL3);


end structural;
