library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity DATAPATH_ZEROSKIP_SINGLECHANNEL is  -- (FOR 128x128 PIXELS)
generic (D: integer:=9; --9
         K: integer:=6; -- 24
         G: integer:=16; -- 16
 	 reg1: integer:=9; --9
         reg2: integer:= 16; --16
	 MEM: integer:= 128 -- 128
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     PIXELS: in std_logic_vector(D-1 downto 0);
     WEIGHT0: in std_logic_vector(K-1 downto 0);
     WEIGHT1: in std_logic_vector(K-1 downto 0); 
     WEIGHT2: in std_logic_vector(K-1 downto 0);
     WEIGHT3: in std_logic_vector(K-1 downto 0);
     WEIGHT4: in std_logic_vector(K-1 downto 0);
     WEIGHT5: in std_logic_vector(K-1 downto 0);
     WEIGHT6: in std_logic_vector(K-1 downto 0);
     WEIGHT7: in std_logic_vector(K-1 downto 0);
     WEIGHT8: in std_logic_vector(K-1 downto 0);
     REGISTERS_ENABLE: in std_logic;
     OM_0_INPUT: in std_logic_vector (reg2-1 downto 0);
     OM_1_INPUT: in std_logic_vector (reg2-1 downto 0);
     OM_0_OUTPUT: out std_logic_vector (reg2-1 downto 0);
     OM_1_OUTPUT: out std_logic_vector (reg2-1 downto 0);
     MAP_RESULT: out std_logic_vector(G-1 downto 0)); 
end DATAPATH_ZEROSKIP_SINGLECHANNEL;    

architecture structural of DATAPATH_ZEROSKIP_SINGLECHANNEL is 

component PE_ZEROSKIPPING is
generic (n: integer; -- 9
         m: integer); --16
port( CLK: in std_logic;
      RESET: in std_logic;
      ZERO_EN: in std_logic;
      ACTIVATION: in std_logic_vector(n-1 downto 0);
      WEIGHT: in std_logic_vector(n-4 downto 0);
      PARTIAL_SUM: in std_logic_vector(m-1 downto 0);
      PARTIAL_RESULT: out std_logic_vector(m-1 downto 0));
end component;

component REGISTER_1bit is
port ( CLK: in std_logic;
       RESET: in std_logic;
       ENABLE: in std_logic;	   
       D : in std_logic;
       Q : out std_logic);
end component;

component REGISTER_nbit is
generic (n: integer);
port ( CLK: in std_logic;
       RESET: in std_logic;
       ENABLE: in std_logic;	   
       D : in std_logic_vector (n-1 downto 0);
       Q : out std_logic_vector (n-1 downto 0));
end component;

component INPUT_ZERO_RECOGNIZER is
generic (n: integer); --9
port( ACTIVATION: in std_logic_vector(n-1 downto 0);
      WEIGHT: in std_logic_vector(n-4 downto 0);
      ZERO: out std_logic);
end component;


-- WIRES 
signal REG1_REG2_ZIR0: std_logic_vector(reg1-1 downto 0);
signal REG2_REG3_PE0: std_logic_vector(reg1-1 downto 0):=(others=>'0');
signal REG3_REG4_ZIR1: std_logic_vector(reg1-1 downto 0);
signal REG4_REG5_PE1: std_logic_vector(reg1-1 downto 0):=(others=>'0');
signal REG5_REG6_ZIR2: std_logic_vector(reg1-1 downto 0);
signal REG6_REG7_PE2: std_logic_vector(reg1-1 downto 0):=(others=>'0');
signal REG7_REG8_ZIR3: std_logic_vector(reg1-1 downto 0);
signal REG8_REG9_PE3: std_logic_vector(reg1-1 downto 0):=(others=>'0');
signal REG9_REG10_ZIR4: std_logic_vector(reg1-1 downto 0);
signal REG10_REG11_PE4: std_logic_vector(reg1-1 downto 0):=(others=>'0');
signal REG11_REG12_ZIR5: std_logic_vector(reg1-1 downto 0);
signal REG12_REG13_PE5: std_logic_vector(reg1-1 downto 0):=(others=>'0');
signal REG13_REG14_ZIR6: std_logic_vector(reg1-1 downto 0);
signal REG14_REG15_PE6: std_logic_vector(reg1-1 downto 0):=(others=>'0');
signal REG15_REG16_ZIR7: std_logic_vector(reg1-1 downto 0);
signal REG16_REG17_PE7: std_logic_vector(reg1-1 downto 0):=(others=>'0');
signal REG17_REG18_ZIR8: std_logic_vector(reg1-1 downto 0);
signal REG18_PE8: std_logic_vector(reg1-1 downto 0);

signal ZIR0_REGZERO0: std_logic;
signal REGZERO0_PE0: std_logic;
signal ZIR1_REGZERO1: std_logic;
signal REGZERO1_PE1: std_logic;
signal ZIR2_REGZERO2: std_logic;
signal REGZERO2_PE2: std_logic;
signal ZIR3_REGZERO3: std_logic;
signal REGZERO3_PE3: std_logic;
signal ZIR4_REGZERO4: std_logic;
signal REGZERO4_PE4: std_logic;
signal ZIR5_REGZERO5: std_logic;
signal REGZERO5_PE5: std_logic;
signal ZIR6_REGZERO6: std_logic;
signal REGZERO6_PE6: std_logic;
signal ZIR7_REGZERO7: std_logic;
signal REGZERO7_PE7: std_logic;
signal ZIR8_REGZERO8: std_logic;
signal REGZERO8_PE8: std_logic;

signal PE0_MACREG0: std_logic_vector(reg2-1 downto 0);
signal PE1_MACREG1: std_logic_vector(reg2-1 downto 0);
signal PE2_MACREG2: std_logic_vector(reg2-1 downto 0);
signal PE3_MACREG3: std_logic_vector(reg2-1 downto 0);
signal PE4_MACREG4: std_logic_vector(reg2-1 downto 0);
signal PE5_MACREG5: std_logic_vector(reg2-1 downto 0);
signal PE6_MACREG6: std_logic_vector(reg2-1 downto 0);
signal PE7_MACREG7: std_logic_vector(reg2-1 downto 0);
signal PE8_MACREG8: std_logic_vector(reg2-1 downto 0);

signal MACREG0_PE1: std_logic_vector(reg2-1 downto 0);
signal MACREG1_PE2: std_logic_vector(reg2-1 downto 0);
--signal MACREG2_OM0: std_logic_vector(reg2-1 downto 0);
--signal OM0_PE3: std_logic_vector(reg2-1 downto 0);
signal MACREG3_PE4: std_logic_vector(reg2-1 downto 0);
signal MACREG4_PE5: std_logic_vector(reg2-1 downto 0);
--signal MACREG5_OM1: std_logic_vector(reg2-1 downto 0);
--signal OM1_PE6: std_logic_vector(reg2-1 downto 0);
signal MACREG6_PE7: std_logic_vector(reg2-1 downto 0);
signal MACREG7_PE8: std_logic_vector(reg2-1 downto 0);


begin

-- UUT 

UUT_REG1: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, PIXELS, REG1_REG2_ZIR0); 

UUT_REG2: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG1_REG2_ZIR0, REG2_REG3_PE0);

UUT_REG3: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG2_REG3_PE0, REG3_REG4_ZIR1);

UUT_REG4: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG3_REG4_ZIR1, REG4_REG5_PE1);

UUT_REG5: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG4_REG5_PE1, REG5_REG6_ZIR2);

UUT_REG6: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG5_REG6_ZIR2, REG6_REG7_PE2);

UUT_REG7: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG6_REG7_PE2, REG7_REG8_ZIR3);

UUT_REG8: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG7_REG8_ZIR3, REG8_REG9_PE3);

UUT_REG9: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG8_REG9_PE3, REG9_REG10_ZIR4);

UUT_REG10: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG9_REG10_ZIR4, REG10_REG11_PE4);

UUT_REG11: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG10_REG11_PE4, REG11_REG12_ZIR5);

UUT_REG12: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG11_REG12_ZIR5, REG12_REG13_PE5);

UUT_REG13: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG12_REG13_PE5, REG13_REG14_ZIR6);

UUT_REG14: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG13_REG14_ZIR6, REG14_REG15_PE6);

UUT_REG15: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG14_REG15_PE6, REG15_REG16_ZIR7);

UUT_REG16: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG15_REG16_ZIR7, REG16_REG17_PE7);

UUT_REG17: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG16_REG17_PE7, REG17_REG18_ZIR8);

UUT_REG18: REGISTER_nbit generic map (n=>reg1) port map (CLK, RESET, REGISTERS_ENABLE, REG17_REG18_ZIR8, REG18_PE8);

UUT_ZIR0: INPUT_ZERO_RECOGNIZER generic map(n=>reg1) port map (REG1_REG2_ZIR0, WEIGHT2, ZIR0_REGZERO0);

UUT_ZIR1: INPUT_ZERO_RECOGNIZER generic map(n=>reg1) port map (REG3_REG4_ZIR1, WEIGHT1, ZIR1_REGZERO1);

UUT_ZIR2: INPUT_ZERO_RECOGNIZER generic map(n=>reg1) port map (REG5_REG6_ZIR2, WEIGHT0, ZIR2_REGZERO2);

UUT_ZIR3: INPUT_ZERO_RECOGNIZER generic map(n=>reg1) port map (REG7_REG8_ZIR3, WEIGHT5, ZIR3_REGZERO3);

UUT_ZIR4: INPUT_ZERO_RECOGNIZER generic map(n=>reg1) port map (REG9_REG10_ZIR4, WEIGHT4, ZIR4_REGZERO4);

UUT_ZIR5: INPUT_ZERO_RECOGNIZER generic map(n=>reg1) port map (REG11_REG12_ZIR5, WEIGHT3, ZIR5_REGZERO5);

UUT_ZIR6: INPUT_ZERO_RECOGNIZER generic map(n=>reg1) port map (REG13_REG14_ZIR6, WEIGHT8, ZIR6_REGZERO6);

UUT_ZIR7: INPUT_ZERO_RECOGNIZER generic map(n=>reg1) port map (REG15_REG16_ZIR7, WEIGHT7, ZIR7_REGZERO7);

UUT_ZIR8: INPUT_ZERO_RECOGNIZER generic map(n=>reg1) port map (REG17_REG18_ZIR8, WEIGHT6, ZIR8_REGZERO8);

UUT_REGZERO0: REGISTER_1bit port map (CLK, RESET, REGISTERS_ENABLE, ZIR0_REGZERO0, REGZERO0_PE0);

UUT_REGZERO1: REGISTER_1bit port map (CLK, RESET, REGISTERS_ENABLE, ZIR1_REGZERO1, REGZERO1_PE1);

UUT_REGZERO2: REGISTER_1bit port map (CLK, RESET, REGISTERS_ENABLE, ZIR2_REGZERO2, REGZERO2_PE2);

UUT_REGZERO3: REGISTER_1bit port map (CLK, RESET, REGISTERS_ENABLE, ZIR3_REGZERO3, REGZERO3_PE3);

UUT_REGZERO4: REGISTER_1bit port map (CLK, RESET, REGISTERS_ENABLE, ZIR4_REGZERO4, REGZERO4_PE4);

UUT_REGZERO5: REGISTER_1bit port map (CLK, RESET, REGISTERS_ENABLE, ZIR5_REGZERO5, REGZERO5_PE5);

UUT_REGZERO6: REGISTER_1bit port map (CLK, RESET, REGISTERS_ENABLE, ZIR6_REGZERO6, REGZERO6_PE6);

UUT_REGZERO7: REGISTER_1bit port map (CLK, RESET, REGISTERS_ENABLE, ZIR7_REGZERO7, REGZERO7_PE7);

UUT_REGZERO8: REGISTER_1bit port map (CLK, RESET, REGISTERS_ENABLE, ZIR8_REGZERO8, REGZERO8_PE8);


UUT_PE0: PE_ZEROSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, REGZERO0_PE0, REG2_REG3_PE0, WEIGHT2, (others=>'0'), PE0_MACREG0);

UUT_PE1: PE_ZEROSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, REGZERO1_PE1, REG4_REG5_PE1, WEIGHT1, MACREG0_PE1, PE1_MACREG1);

UUT_PE2: PE_ZEROSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, REGZERO2_PE2, REG6_REG7_PE2, WEIGHT0, MACREG1_PE2, PE2_MACREG2);

UUT_PE3: PE_ZEROSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, REGZERO3_PE3, REG8_REG9_PE3, WEIGHT5, OM_0_INPUT, PE3_MACREG3);

UUT_PE4: PE_ZEROSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, REGZERO4_PE4, REG10_REG11_PE4, WEIGHT4, MACREG3_PE4, PE4_MACREG4);

UUT_PE5: PE_ZEROSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, REGZERO5_PE5, REG12_REG13_PE5, WEIGHT3, MACREG4_PE5, PE5_MACREG5);

UUT_PE6: PE_ZEROSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, REGZERO6_PE6, REG14_REG15_PE6, WEIGHT8, OM_1_INPUT, PE6_MACREG6);

UUT_PE7: PE_ZEROSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, REGZERO7_PE7, REG16_REG17_PE7, WEIGHT7, MACREG6_PE7, PE7_MACREG7);

UUT_PE8: PE_ZEROSKIPPING generic map(n=>reg1, m=>reg2) port map(CLK, RESET, REGZERO8_PE8, REG18_PE8, WEIGHT6, MACREG7_PE8, PE8_MACREG8);


UUT_MACREG0: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE0_MACREG0, MACREG0_PE1);

UUT_MACREG1: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE1_MACREG1, MACREG1_PE2);

UUT_MACREG2: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE2_MACREG2, OM_0_OUTPUT);

UUT_MACREG3: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE3_MACREG3, MACREG3_PE4);

UUT_MACREG4: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE4_MACREG4, MACREG4_PE5);

UUT_MACREG5: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE5_MACREG5, OM_1_OUTPUT);

UUT_MACREG6: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE6_MACREG6, MACREG6_PE7);

UUT_MACREG7: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE7_MACREG7, MACREG7_PE8);

UUT_MACREG8: REGISTER_nbit generic map (n=>reg2) port map(CLK, RESET, REGISTERS_ENABLE, PE8_MACREG8, MAP_RESULT);


end structural;
