library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity OFF_CHIP_CONTROLLER is  -- (FOR 128x128 PIXELS)
generic (N_ACTS: integer:=16384; -- 16384
         M: integer:=128; --128
         l: integer:=126; --126
         mm: integer:=125; --125
         n: integer:=277; -- 276
         K: integer:=16403  -- 16402   
         );         
port(CLK: in std_logic;
     RESET: in std_logic;
     START: in std_logic;
     ACTIVATIONS_MEMORY_OE: out std_logic;
     WEIGHTS_MEMORY_OE: out std_logic;
     OFFM_WE: out std_logic;
     OUT_MAP: out std_logic);
     end OFF_CHIP_CONTROLLER;   

architecture behavioural of OFF_CHIP_CONTROLLER is

signal CNT_CLK: integer:= 0;

begin

-- COUNTER PROCESS TO INDICATE CLK CYCLES
CNT_PROCESS: process (START,CLK,RESET)
begin

if RESET = '1' then
CNT_CLK <= 0;
elsif CLK'event and CLK = '1' then
if START = '1' then
CNT_CLK <= CNT_CLK + 1;
end if;
end if;
end process;

-- LOADING WEIGHTS AND ACTIVATIONS

ACTS_WEIGHTS_PROCESS: process(START,CLK,RESET)
begin

if RESET = '1' then            -- GO TO IDLE
WEIGHTS_MEMORY_OE <= '0';
ACTIVATIONS_MEMORY_OE <= '0';
elsif CLK'event and CLK = '1' then
if START = '1' then
if CNT_CLK = 0 then
   WEIGHTS_MEMORY_OE <= '1';
   else
   WEIGHTS_MEMORY_OE <= '0';
   end if;

if CNT_CLK < N_ACTS then
   ACTIVATIONS_MEMORY_OE <= '1';
   else 
   ACTIVATIONS_MEMORY_OE <= '0';
   end if;
   
end if;
end if;

end process;

-- OFF_MAP WRITING

OFF_MAP_WRITING: process (START,CLK, RESET)
variable i: integer:= n;
begin

if RESET = '1' then -- GO TO IDLE
OFFM_WE <= '0';
elsif (CLK'event and CLK = '1') then
	if START = '1' then  
	 if CNT_CLK >= i and CNT_CLK <= (i+mm) and CNT_CLK < K then
	    OFFM_WE <= '1';
		else
		OFFM_WE <= '0';
        end if;
		
	if CNT_CLK = (i+l) then
           i:= i+M;
        end if;
 
end if;
end if;
end process;

----------------------------------------******

----------------------------------------******

-- MANAGING OUTPUT

OFF_MAP_READING_PROCESS: process (START,CLK, RESET)

begin

if RESET = '1' then -- GO TO IDLE
OUT_MAP <= '0';
elsif (CLK'event and CLK = '1') then
	if START = '1' then  
if CNT_CLK = K then
OUT_MAP <= '1';
else
OUT_MAP <= '0';
end if;
 
end if;
end if;
end process;

----------------------------------------******

end behavioural;
	