//`timescale 1ns

module tb_CONVOLUTER_ZEROSKIP_TOP ();
   
   wire END_SIM_i;
   wire END_SIM1_i;
   wire END_SIM2_i;
   wire END_SIM3_i;
   wire START_i;
   wire CLK_i;
   wire RST_n_i;
   wire EN_AM_R_i;
   wire EN_WM_R_i;
   wire [26:0] PIX_i;
   wire [17:0] W0_i;
   wire [17:0] W1_i;
   wire [17:0] W2_i;
   wire [17:0] W3_i;
   wire [17:0] W4_i;
   wire [17:0] W5_i;
   wire [17:0] W6_i;
   wire [17:0] W7_i;
   wire [17:0] W8_i;
   wire [15:0] OM0_IN_CH1;
   wire [15:0] OM1_IN_CH1;
   wire [15:0] OM0_O_CH1;
   wire [15:0] OM1_O_CH1;
   wire [15:0] OM0_IN_CH2;
   wire [15:0] OM1_IN_CH2;
   wire [15:0] OM0_O_CH2;
   wire [15:0] OM1_O_CH2;
   wire [15:0] OM0_IN_CH3;
   wire [15:0] OM1_IN_CH3;
   wire [15:0] OM0_O_CH3;
   wire [15:0] OM1_O_CH3;
   wire        OM0_WE;
   wire        OM1_WE;
   wire        OM0_OE;
   wire        OM1_OE;
   wire EN_OFF_W_i;
   wire EN_OFF_R_i;
   wire [15:0] M_CH_1_i;
   wire [15:0] M_CH_2_i;
   wire [15:0] M_CH_3_i;

    initial begin
      $read_lib_saif("../saif/uk65lscllmvbbr_120c25_tc.saif");
      $set_gate_level_monitoring("on");
      $set_toggle_region(CVA_ZS);
      $toggle_start();
      end
 

     SIMULATION SIM(  .END_SIM_1(END_SIM1_i), 
                      .END_SIM_2(END_SIM2_i),
	              .END_SIM_3(END_SIM3_i),
                      .START(START_i),
                      .END_SIM(END_SIM_i));

      clk_gen CG( .END_SIM(END_SIM_i), 
                  .CLK(CLK_i),
	          .RST_n(RST_n_i));
		  
	ACTIVATIONS_MEMORY_ROM A_MEM(.CLK(CLK_i),
                               .RESET(RST_n_i),
                               .OE(EN_AM_R_i),
                               .DATA_OUTPUT(PIX_i));
	
	WEIGHTS_MEMORY_ROM  W_MEM(.CLK(CLK_i),
                              .RESET(RST_n_i),
                              .OE(EN_WM_R_i),
                              .W_0(W0_i),
			      .W_1(W1_i),
			      .W_2(W2_i),
                              .W_3(W3_i),
		              .W_4(W4_i),
	                      .W_5(W5_i),
			      .W_6(W6_i),
		              .W_7(W7_i),
			      .W_8(W8_i));

      	OFF_CHIP_CONTROLLER OFCONT(.CLK(CLK_i),
                                   .RESET(RST_n_i),
                                   .START(START_i),
                                   .ACTIVATIONS_MEMORY_OE(EN_AM_R_i),
                                   .WEIGHTS_MEMORY_OE(EN_WM_R_i),
                                   .OFFM_WE(EN_OFF_W_i),
                                   .OUT_MAP(EN_OFF_R_i));

						  
   CONV_ACCELERATOR_ZEROSKIP CVA_ZS(.CLK(CLK_i), 
	                .RESET(RST_n_i),
                        .START(START_i),
		        .PIXELS(PIX_i),
		        .WEIGHT0(W0_i),
		        .WEIGHT1(W1_i),
		        .WEIGHT2(W2_i),
		        .WEIGHT3(W3_i),
		        .WEIGHT4(W4_i),
		        .WEIGHT5(W5_i),
		        .WEIGHT6(W6_i),
		        .WEIGHT7(W7_i),
		        .WEIGHT8(W8_i),
			.OM0WE(OM0_WE),
                        .OM0OE(OM0_OE),
                        .OM1WE(OM1_WE),
                        .OM1OE(OM1_OE),
                        .OM0_INPUT_CHANNEL1(OM0_IN_CH1),
                        .OM1_INPUT_CHANNEL1(OM1_IN_CH1),
                        .OM0_OUTPUT_CHANNEL1(OM0_O_CH1),
                        .OM1_OUTPUT_CHANNEL1(OM1_O_CH1),
                        .OM0_INPUT_CHANNEL2(OM0_IN_CH2),
                        .OM1_INPUT_CHANNEL2(OM1_IN_CH2),
                        .OM0_OUTPUT_CHANNEL2(OM0_O_CH2),
                        .OM1_OUTPUT_CHANNEL2(OM1_O_CH2),
                        .OM0_INPUT_CHANNEL3(OM0_IN_CH3),
                        .OM1_INPUT_CHANNEL3(OM1_IN_CH3),
                        .OM0_OUTPUT_CHANNEL3(OM0_O_CH3),
                        .OM1_OUTPUT_CHANNEL3(OM1_O_CH3),
                        .MAP_RESULT_CHANNEL1(M_CH_1_i),
	                .MAP_RESULT_CHANNEL2(M_CH_2_i),
	                .MAP_RESULT_CHANNEL3(M_CH_3_i));
                        
      ON_CHIP_MEMORY_RAM  #(.m(128),
                    .n(16))
    
      OM0_CH1 (.CLK(CLK_i),
				  .RESET(RST_n_i),
				  .WE(OM0_WE),
				  .OE(OM0_OE),
				  .DATA_INPUT(OM0_O_CH1),
				  .DATA_OUTPUT(OM0_IN_CH1));
   
     ON_CHIP_MEMORY_RAM  #(.m(128),
                   .n(16))
     
     
     OM0_CH2 (.CLK(CLK_i),
				  .RESET(RST_n_i),
				  .WE(OM0_WE),
				  .OE(OM0_OE),
				  .DATA_INPUT(OM0_O_CH2),
				  .DATA_OUTPUT(OM0_IN_CH2));

     ON_CHIP_MEMORY_RAM #(.m(128),
                   .n(16))
     
     OM0_CH3 (.CLK(CLK_i),
				  .RESET(RST_n_i),
				  .WE(OM0_WE),
				  .OE(OM0_OE),
				  .DATA_INPUT(OM0_O_CH3),
				  .DATA_OUTPUT(OM0_IN_CH3));

     ON_CHIP_MEMORY_RAM  #(.m(128),
                   .n(16))
     
     
     OM1_CH1 (.CLK(CLK_i),
				  .RESET(RST_n_i),
				  .WE(OM1_WE),
				  .OE(OM1_OE),
				  .DATA_INPUT(OM1_O_CH1),
				  .DATA_OUTPUT(OM1_IN_CH1));

     ON_CHIP_MEMORY_RAM  #(.m(128),
                   .n(16))
     
     
     OM1_CH2 (.CLK(CLK_i),
				  .RESET(RST_n_i),
				  .WE(OM1_WE),
				  .OE(OM1_OE),
				  .DATA_INPUT(OM1_O_CH2),
				  .DATA_OUTPUT(OM1_IN_CH2));

     ON_CHIP_MEMORY_RAM  #(.m(128),
                   .n(16))
     
     OM1_CH3 (.CLK(CLK_i),
				  .RESET(RST_n_i),
				  .WE(OM1_WE),
				  .OE(OM1_OE),
				  .DATA_INPUT(OM1_O_CH3),
				  .DATA_OUTPUT(OM1_IN_CH3));


   OFF_CHIP_MEMORY_DRAM_CHANNEL1 OFF_CH1(.CLK(CLK_i),
	                                 .RESET(RST_n_i),
					 .WE(EN_OFF_W_i),
					 .OE(EN_OFF_R_i),
					 .DATA_INPUT(M_CH_1_i),
                                         .END_SIM(END_SIM1_i));
										 
   OFF_CHIP_MEMORY_DRAM_CHANNEL2 OFF_CH2(.CLK(CLK_i),
	                                 .RESET(RST_n_i),
					 .WE(EN_OFF_W_i),
					 .OE(EN_OFF_R_i),
					 .DATA_INPUT(M_CH_2_i),
                                         .END_SIM(END_SIM2_i));
										 
   OFF_CHIP_MEMORY_DRAM_CHANNEL3 OFF_CH3(.CLK(CLK_i),
	                                 .RESET(RST_n_i),
					 .WE(EN_OFF_W_i),
					 .OE(EN_OFF_R_i),
					 .DATA_INPUT(M_CH_3_i),
                                         .END_SIM(END_SIM3_i));


   	always @ (END_SIM_i) begin
      if(END_SIM_i) begin
	 $toggle_stop();
         $toggle_report("../saif/CONV_ACCELERATOR_ZEROSKIP_back.saif", 1.0e-11, "tb_CONVOLUTER_ZEROSKIP_TOP.CVA_ZS");
      end
	end
   
endmodule
	


		   